#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef JFB_ENABLE_SPLASH_SCREEN
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "read-bmp.h"
#include "read-png.h"
#include "splash.h"
#include "message.h"
#include "fbcommon.h"

TSplash gSplash;

void
splash_exit(TSplash *s)
{
	if(s->bitmap != NULL) {
		free(s->bitmap);
	}
}

void
splash_bitmap_simple_resize(TSplash *s, int dw, int dh)
{
	int rw, rh;
	int x, y;
	u_char *dst;

	if(s->use_splash == 0)
		return;
	if(s->bitmap == NULL)
		return;

	dst = (u_char *)malloc(dh * dw * 3 * sizeof(u_char));
	for(y = 0; y < dh; y++) {
		rh = y * s->height / dh;
		for(x = 0; x < dw; x++) {
			rw = x * s->width / dw;
			SPLASH_GETP(dst, dw, x, y, 0) = SPLASH_GETP(s->bitmap, s->width, rw, rh, 0);
			SPLASH_GETP(dst, dw, x, y, 1) = SPLASH_GETP(s->bitmap, s->width, rw, rh, 1);
			SPLASH_GETP(dst, dw, x, y, 2) = SPLASH_GETP(s->bitmap, s->width, rw, rh, 2);
		}
	}
	free(s->bitmap);
	s->bitmap = dst;
	s->width = dw;
	s->height = dh;
}

void
splash_print_status(TSplash *s)
{
	if(s->use_splash != 0) {
		print_message("(UXP Splash) Splash enabled\n");
	}
	else {
		print_message("(UXP Splash) Disabled\n");
	}
}


void
splash_init(TSplash *s, char *filename)
{
	FILE *fp;
	int filename_len;

	filename_len = strlen(filename);
	s->bitmap = NULL;
	s->use_splash = 0;
	s->width = 0;
	s->height = 0;
	s->tint = 0;

	if(gFramebuffer.cap.bitsPerPixel != 15 && gFramebuffer.cap.bitsPerPixel != 16 &&
		gFramebuffer.cap.bitsPerPixel != 24 && gFramebuffer.cap.bitsPerPixel != 32) {
		print_message("UXP Splash) Framebuffer is not 15, 16, 24 or 32 bpp\n");
		return;
	}

	print_message("(UXP Splash) reading file %s\n", filename);
	if(filename_len > 4) {
		if(!strcmp(filename + filename_len - 4, ".bmp")) {
			if((fp = fopen(filename, "r")) != NULL) {
				s->bitmap = read_bmp_file(fp, &(s->width), &(s->height));
				fclose(fp);
				if(s->bitmap != NULL) 
					s->use_splash = 1;
			}
			else {
				print_message("(UXP Splash) ERROR: cannot read file\n");
			}
		}
#ifdef JFB_WITH_LIBPNG
		else if(!strcmp(filename + filename_len - 4, ".png")) {
			if((fp = fopen(filename, "r")) != NULL) {
				s->bitmap = read_png_file(fp, &(s->width), &(s->height));
				fclose(fp);
				if(s->bitmap != NULL)
					s->use_splash = 1;
			}
			else {
				print_message("(UXP Splash) ERROR: cannot read file\n");
			}
		}
#endif
	}
}

void
splash_bitmap_tint(TSplash *s, int tint)
{
	int x, y;
	if(s->use_splash == 0)
		return;
	if(s->bitmap == NULL)
		return;
	tint = 10 - tint;
	if(tint >= 10)
		return;
	if(tint < 0)
		tint = 0;
	for(y = 0; y < s->height; y++) {
		for(x = 0; x < s->width; x++) {
			SPLASH_GETP(gSplash.bitmap, gSplash.width, x, y, 0) *= (tint / 10.0);
			SPLASH_GETP(gSplash.bitmap, gSplash.width, x, y, 1) *= (tint / 10.0);
			SPLASH_GETP(gSplash.bitmap, gSplash.width, x, y, 2) *= (tint / 10.0);
		}
	}
}
#endif /* JFB_ENABLE_SPLASH_SCREEN */
